@@ -42,16 +42,16 @@ is_time_present <- function(input) {
   grepl(get_time_pattern(), input)
 }
 
+# returns a vector of length of input
 is_tz_present <- function(input) {
-
-  any(
-    c(
-      grepl(get_tz_pattern_z(), input),
-      grepl(get_tz_pattern_hh(), input),
-      grepl(get_tz_pattern_hh_mm(), input),
-      grepl(get_tz_pattern_hhmm(), input)
-    )
+  regex <- paste(
+    get_tz_pattern_z(),
+    get_tz_pattern_hh(),
+    get_tz_pattern_hh_mm(),
+    get_tz_pattern_hhmm(),
+    sep = "|"
   )
+  grepl(regex, input)
 }
 
 is_long_tzid_present <- function(input) {
@@ -99,8 +99,7 @@ normalize_long_tzid <- function(long_tzid) {
   if (long_tzid %in% tz_name_resolution$tz_alt) {
 
     long_tzid <-
-      tz_name_resolution[
-        tz_name_resolution$tz_alt == long_tzid, ][["tz_canonical"]]
+      tz_name_resolution$tz_canonical[tz_name_resolution$tz_alt == long_tzid]
   }
 
   long_tzid
@@ -118,17 +117,18 @@ long_tzid_to_tz_str <- function(long_tzid, input_dt) {
     return("+0000")
   }
 
-  tzdb_entries_tzid <- tzdb[tzdb$zone_name == long_tzid, ]
+  tzdb_entries_tzid <- tzdb[
+    tzdb$zone_name == long_tzid, c("date_start", "gmt_offset_h")]
 
-  if (nrow(tzdb_entries_tzid) < 1) {
-    return(NA_character_)
+  if (nrow(tzdb_entries_tzid) == 0L) {
+    return(NULL)
   }
 
   input_date <- as.Date(input_dt)
 
-  tzdb_idx <- rle(!(tzdb_entries_tzid$date_start < input_date))$lengths[1]
+  tzdb_idx <- rle(tzdb_entries_tzid$date_start >= input_date)$lengths[1]
 
-  tz_offset <- tzdb_entries_tzid[tzdb_idx, ]$gmt_offset_h
+  tz_offset <- tzdb_entries_tzid[[tzdb_idx, "gmt_offset_h"]]
 
   minutes <- formatC(round((abs(tz_offset) %% 1) * 60, 0), width = 2, flag = "0")
   hours <- formatC(trunc(abs(tz_offset)), width = 2, flag = "0")
@@ -139,7 +139,7 @@ long_tzid_to_tz_str <- function(long_tzid, input_dt) {
 
 get_tz_str <- function(input) {
 
-  if (!is_tz_present(input = input)) {
+  if (is.null(input) || !is_tz_present(input = input)) {
     return("")
   }
 
@@ -189,9 +189,7 @@ get_tz_offset_val_from_tz_str <- function(tz_str) {
 
 get_tz_offset_val <- function(input, tz_str = NULL) {
 
-  if (is.null(tz_str)) {
-    tz_str <- get_tz_str(input = input)
-  }
+  tz_str <- tz_str %||% get_tz_str(input = input)
 
   get_tz_offset_val_from_tz_str(tz_str = tz_str)
 }
@@ -232,10 +230,6 @@ which_tz_pattern <- function(input) {
     return("z")
   }
 
-  if (grepl(get_tz_pattern_hh(), input)) {
-    tz_pattern <- "hh"
-  }
-
   if (grepl(get_tz_pattern_hh_mm(), input)) {
     return("hh_mm")
   }
@@ -244,8 +238,8 @@ which_tz_pattern <- function(input) {
     return("hhmm")
   }
 
-  if (exists("tz_pattern")) {
-    return(tz_pattern)
+  if (grepl(get_tz_pattern_hh(), input)) {
+    return("hh")
   }
 
   NA
@@ -257,9 +251,8 @@ which_iana_pattern <- function(input) {
     return("wrapped")
   } else if (grepl(paste0(get_tz_pattern(), get_attached_iana_pattern()), input)) {
     return("attached")
-  } else {
-    return(NA_character_)
   }
+  NA_character_
 }
 
 get_tz_offset <- function(input) {
@@ -315,14 +308,14 @@ get_localized_exemplar_city <- function(
 
     # Get localized variant of 'Unknown City'
     if (yield_unknown) {
-      return(tz_exemplar[tz_exemplar$locale == locale, ][["Unknown"]])
+      return(tz_exemplar[tz_exemplar$locale == locale, "Unknown"])
     } else {
       return(NA_character_)
     }
   }
 
   exemplar_city_localized <-
-    tz_exemplar[tz_exemplar$locale == locale, ][[exemplar_city]]
+    tz_exemplar[tz_exemplar$locale == locale, exemplar_city]
 
   exemplar_city_localized
 }
@@ -330,18 +323,18 @@ get_localized_exemplar_city <- function(
 # The short specific non-location format (e.g., 'PST') from a `long_tzid`
 get_tz_short_specific <- function(long_tzid, input_dt) {
 
+  tzdb_entries_tzid <- tzdb[tzdb$zone_name == long_tzid, ]
+  if (nrow(tzdb_entries_tzid) == 0) {
+    return(NA_character_)
+  }
+
   input_date <- as.Date(input_dt)
 
-  tzdb_entries_tzid <- tzdb[tzdb$zone_name == long_tzid, ]
-
-  tzdb_idx <- rle(!(tzdb_entries_tzid$date_start < input_date))$lengths[1]
-
-  tz_short_specific <- tzdb_entries_tzid[tzdb_idx, ]$abbrev
+  tzdb_idx <- rle(tzdb_entries_tzid$date_start >= input_date)$lengths[1]
 
   # TODO: add check to ensure that the `abbrev` value is a valid
   # short specific non-location time zone
-
-  tz_short_specific
+  tzdb_entries_tzid[tzdb_idx, "abbrev"]
 }
 
 # The long specific non-location format (e.g., 'Pacific Standard Time') from
@@ -352,11 +345,11 @@ get_tz_long_specific <- function(long_tzid, input_dt, locale) {
 
   tzdb_entries_tzid <- tzdb[tzdb$zone_name == long_tzid, ]
 
-  if (nrow(tzdb_entries_tzid) < 1) {
+  if (nrow(tzdb_entries_tzid) == 0L) {
     return(NA_character_)
   }
 
-  tzdb_idx <- rle(!(tzdb_entries_tzid$date_start < input_date))$lengths[1]
+  tzdb_idx <- rle(tzdb_entries_tzid$date_start >= input_date)$lengths[1]
 
   tzdb_entries_tzid_ln <- tzdb_entries_tzid[tzdb_idx, ]
 
@@ -379,7 +372,7 @@ get_tz_long_specific <- function(long_tzid, input_dt, locale) {
   # Get the row of the `tz_metazone_names` table based on the supplied locale
   tz_metazone_names_row <- tz_metazone_names[tz_metazone_names$locale == locale, ]
 
-  if (nrow(tz_metazone_names_row) < 1) {
+  if (nrow(tz_metazone_names_row) == 0) {
     return(NA_character_)
   }
 
@@ -432,13 +425,8 @@ get_tz_non_location <- function(
     type
 ) {
 
-  if (!(short_long %in% c("long", "short"))) {
-    stop("The `short_long` keyword should either be 'long' or 'short'.")
-  }
-
-  if (!(type %in% c("generic", "standard", "daylight"))) {
-    stop("The `short_long` keyword should either be 'long' or 'short'.")
-  }
+  short_long <- match.arg(short_long, c("long", "short"))
+  type <- match.arg(type, c("generic", "standard", "daylight"))
 
   # If the supplied `long_tzid` value is NA, return NA
   if (is.na(long_tzid)) {
@@ -471,7 +459,7 @@ get_tz_non_location <- function(
     tz_name <- tz_metazone_names_entry[[available_items]]
   } else if (target_item %in% available_items) {
     tz_name <- tz_metazone_names_entry[[target_item]]
-  } else if (short_long == "short" && !has_short_items) {
+  } else if (!has_short_items && short_long == "short") {
     if (any(grepl(type, available_items))) {
       tz_name <- tz_metazone_names_entry[[paste0("long.", type)]]
     } else {
@@ -493,32 +481,30 @@ long_tz_id_to_metazone_long_id <- function(long_tzid) {
     if (long_tzid %in% unique(tz_name_resolution$tz_canonical)) {
 
       alt_names <-
-        tz_name_resolution[tz_name_resolution$tz_canonical == long_tzid, ][["tz_alt"]]
+        tz_name_resolution$tz_alt[tz_name_resolution$tz_canonical == long_tzid]
 
-      if (any(alt_names %in% tz_metazone_users$canonical_tz_name)) {
-
-        long_tzid <- alt_names[1]
-
-      } else {
+      if (!any(alt_names %in% tz_metazone_users$canonical_tz_name)) {
         return(NA_character_)
       }
+      long_tzid <- alt_names[1]
 
     } else {
       return(NA_character_)
     }
   }
 
+  rows <- which(tz_metazone_users$canonical_tz_name == long_tzid)
   tz_metazone_users_rows <-
-    tz_metazone_users[tz_metazone_users$canonical_tz_name == long_tzid, ]
+    tz_metazone_users[[rows, "metazone_long_id"]]
 
   # Return NA if number of rows in `tz_metazone_users_rows` is zero
-  if (nrow(tz_metazone_users_rows) < 1) {
+  if (length(tz_metazone_users_rows) == 0) {
     return(NA_character_)
   }
 
   # TODO: develop routine to further filter multirow `tz_metazone_users_rows`
   # to a single row based on `locale`; for now, obtain the first metazone
-  metazone <- tz_metazone_users_rows[1, ][["metazone_long_id"]]
+  metazone <- tz_metazone_users_rows[1]
 
   metazone
 }
@@ -570,25 +556,23 @@ get_iana_tz <- function(input) {
 
   iana_pattern <- which_iana_pattern(input)
 
-  switch(
+  value <- switch(
     iana_pattern,
     wrapped = {
       tz_name <-
         gsub(
           paste0(".*", get_time_pattern(), get_tz_pattern()),
           "", input)
-      tz_name <- gsub("(\\(|\\[|\\)|\\])", "", tz_name)
+      gsub("(\\(|\\[|\\)|\\])", "", tz_name)
     },
-    attached = {
-      tz_name <-
-        gsub(
-          paste0(".*", get_time_pattern(), get_tz_pattern(), "(\\s-\\s|\\s|\\/|\\|)"),
-          "", input)
-    })
-
-  if (!exists("tz_name")) {
-    return(NA_character_)
-  }
+    attached = gsub(
+      paste0(".*", get_time_pattern(), get_tz_pattern(), "(\\s-\\s|\\s|\\/|\\|)"),
+      "",
+      input
+    ),
+    # default
+    NA_character_
+   )
 
   tz_name
 }
